@@ -482,6 +482,23 @@ function environment_check($version, $env_select) {
     $custom_results = environment_custom_checks($version, $env_select);
     $results = array_merge($results, $custom_results);
 
+    // Locate any installed plugins belonging to deleted plugin types and block the install/upgrade process until they are removed.
+    // Plugins on disk which aren't installed and which are either deprecated or deleted will be ignored by install/upgrade anyway,
+    // so are not checked here.
+    $pluginman = \core_plugin_manager::instance();
+    foreach (core_component::get_deleted_plugin_types() as $plugintype => $dir) {
+        foreach ($pluginman->get_installed_plugins($plugintype) as $name => $version) {
+            $plugin = $plugintype . '_' . $name;
+
+            $result = new environment_results('custom_check');
+            $result->setInfo('Deleted plugin detected');
+            $result->setFeedbackStr(['deletedplugintypesdetected', 'admin', $plugin]);
+            $result->setStatus(false);
+            $result->plugin = $plugin;
+            $results[] = $result;
+        }
+    }
+
     // Always use the plugin directory version of environment.xml,
     // add-on developers need to keep those up-to-date with future info.
     foreach (core_component::get_plugin_types() as $plugintype => $unused) {
@@ -895,7 +912,7 @@ function environment_check_php($version, $env_select) {
  * Looks for buggy PCRE implementation, we need unicode support in Moodle...
  * @param string $version xml version we are going to use to test this server
  * @param int|string $env_select one of ENV_SELECT_NEWER | ENV_SELECT_DATAROOT | ENV_SELECT_RELEASE decide xml to use. String means plugin name.
- * @return stdClass results encapsulated in one environment_result object, null if irrelevant
+ * @return ?environment_results results encapsulated in one environment_result object, null if irrelevant
  */
 function environment_check_pcre_unicode($version, $env_select) {
     $result = new environment_results('pcreunicode');
@@ -1103,8 +1120,8 @@ function environment_check_database($version, $env_select) {
  * such bypass functions are able to directly handling the result object
  * although it should be only under exceptional conditions.
  *
- * @param string xmldata containing the bypass data
- * @param object result object to be updated
+ * @param array $xml xml containing the bypass data
+ * @param environment_results $result object to be updated
  * @return void
  */
 function process_environment_bypass($xml, &$result) {
@@ -1144,8 +1161,8 @@ function process_environment_bypass($xml, &$result) {
  * such restrict functions are able to directly handling the result object
  * although it should be only under exceptional conditions.
  *
- * @param string xmldata containing the restrict data
- * @param object result object to be updated
+ * @param array $xml xmldata containing the restrict data
+ * @param environment_results $result object to be updated
  * @return void
  */
 function process_environment_restrict($xml, &$result) {
@@ -1181,8 +1198,8 @@ function process_environment_restrict($xml, &$result) {
  *
  * @uses INCORRECT_FEEDBACK_FOR_REQUIRED
  * @uses INCORRECT_FEEDBACK_FOR_OPTIONAL
- * @param string xmldata containing the feedback data
- * @param object reult object to be updated
+ * @param array $xml xmldata containing the feedback data
+ * @param environment_results $result object to be updated
  */
 function process_environment_messages($xml, &$result) {
 
@@ -1551,7 +1568,7 @@ function get_level($element) {
         $level = $element['@']['level'];
         if (!in_array($level, ['required', 'optional', 'recommended'])) {
             debugging('The level of a check in the environment.xml file must be "required", "optional" or "recommended".',
-                DEBUG_DEVELOPER);
+                    DEBUG_DEVELOPER);
             $level = 'required';
         }
     } else {
@@ -1699,3 +1716,14 @@ function restrict_php_version_82($result) {
 function restrict_php_version_83($result) {
     return restrict_php_version($result, '8.3');
 }
+
+/**
+ * Check if the current PHP version is greater than or equal to
+ * PHP version 8.4
+ *
+ * @param \environment_results $result an environment_results instance
+ * @return bool result of version check
+ */
+function restrict_php_version_84(environment_results $result) {
+    return restrict_php_version($result, '8.4');
+}
