@@ -58,9 +58,11 @@ require_once($CFG->libdir . '/grade/grade_outcome.php');
  * @param int    $itemnumber Most probably 0. Modules can use other numbers when having more than one grade for each user
  * @param mixed  $grades Grade (object, array) or several grades (arrays of arrays or objects), NULL if updating grade_item definition only
  * @param mixed  $itemdetails Object or array describing the grading item, NULL if no change
+ * @param bool   $isbulkupdate If bulk grade update is happening.
  * @return int Returns GRADE_UPDATE_OK, GRADE_UPDATE_FAILED, GRADE_UPDATE_MULTIPLE or GRADE_UPDATE_ITEM_LOCKED
  */
-function grade_update($source, $courseid, $itemtype, $itemmodule, $iteminstance, $itemnumber, $grades=NULL, $itemdetails=NULL) {
+function grade_update($source, $courseid, $itemtype, $itemmodule, $iteminstance, $itemnumber, $grades = null,
+        $itemdetails = null, $isbulkupdate = false) {
     global $USER, $CFG, $DB;
 
     // only following grade_item properties can be changed in this function
@@ -76,22 +78,20 @@ function grade_update($source, $courseid, $itemtype, $itemmodule, $iteminstance,
         return GRADE_UPDATE_FAILED;
     }
 
-    if (!$grade_items = grade_item::fetch_all($params)) {
+    if (!$gradeitems = grade_item::fetch_all($params)) {
         // create a new one
-        $grade_item = false;
-
-    } else if (count($grade_items) == 1){
-        $grade_item = reset($grade_items);
-        unset($grade_items); //release memory
-
+        $gradeitem = false;
+    } else if (count($gradeitems) == 1) {
+        $gradeitem = reset($gradeitems);
+        unset($gradeitems); // Release memory.
     } else {
         debugging('Found more than one grade item');
         return GRADE_UPDATE_MULTIPLE;
     }
 
     if (!empty($itemdetails['deleted'])) {
-        if ($grade_item) {
-            if ($grade_item->delete($source)) {
+        if ($gradeitem) {
+            if ($gradeitem->delete($source)) {
                 return GRADE_UPDATE_OK;
             } else {
                 return GRADE_UPDATE_FAILED;
@@ -102,7 +102,7 @@ function grade_update($source, $courseid, $itemtype, $itemmodule, $iteminstance,
 
 /// Create or update the grade_item if needed
 
-    if (!$grade_item) {
+    if (!$gradeitem) {
         if ($itemdetails) {
             $itemdetails = (array)$itemdetails;
 
@@ -126,11 +126,11 @@ function grade_update($source, $courseid, $itemtype, $itemmodule, $iteminstance,
                 $params[$k] = $v;
             }
         }
-        $grade_item = new grade_item($params);
-        $grade_item->insert();
+        $gradeitem = new grade_item($params);
+        $gradeitem->insert(null, $isbulkupdate);
 
     } else {
-        if ($grade_item->is_locked()) {
+        if ($gradeitem->is_locked()) {
             // no notice() here, test returned value instead!
             return GRADE_UPDATE_ITEM_LOCKED;
         }
@@ -144,33 +144,33 @@ function grade_update($source, $courseid, $itemtype, $itemmodule, $iteminstance,
                     continue;
                 }
                 if (in_array($k, $floats)) {
-                    if (grade_floats_different($grade_item->{$k}, $v)) {
-                        $grade_item->{$k} = $v;
+                    if (grade_floats_different($gradeitem->{$k}, $v)) {
+                        $gradeitem->{$k} = $v;
                         $update = true;
                     }
 
                 } else {
-                    if ($grade_item->{$k} != $v) {
-                        $grade_item->{$k} = $v;
+                    if ($gradeitem->{$k} != $v) {
+                        $gradeitem->{$k} = $v;
                         $update = true;
                     }
                 }
             }
             if ($update) {
-                $grade_item->update();
+                $gradeitem->update(null, $isbulkupdate);
             }
         }
     }
 
 /// reset grades if requested
     if (!empty($itemdetails['reset'])) {
-        $grade_item->delete_all_grades('reset');
+        $gradeitem->delete_all_grades('reset');
         return GRADE_UPDATE_OK;
     }
 
 /// Some extra checks
     // do we use grading?
-    if ($grade_item->gradetype == GRADE_TYPE_NONE) {
+    if ($gradeitem->gradetype == GRADE_TYPE_NONE) {
         return GRADE_UPDATE_OK;
     }
 
@@ -208,12 +208,12 @@ function grade_update($source, $courseid, $itemtype, $itemmodule, $iteminstance,
     $count = count($grades);
     if ($count > 0 and $count < 200) {
         list($uids, $params) = $DB->get_in_or_equal(array_keys($grades), SQL_PARAMS_NAMED, $start='uid');
-        $params['gid'] = $grade_item->id;
+        $params['gid'] = $gradeitem->id;
         $sql = "SELECT * FROM {grade_grades} WHERE itemid = :gid AND userid $uids";
 
     } else {
         $sql = "SELECT * FROM {grade_grades} WHERE itemid = :gid";
-        $params = array('gid'=>$grade_item->id);
+        $params = array('gid' => $gradeitem->id);
     }
 
     $rs = $DB->get_recordset_sql($sql, $params);
@@ -221,7 +221,7 @@ function grade_update($source, $courseid, $itemtype, $itemmodule, $iteminstance,
     $failed = false;
 
     while (count($grades) > 0) {
-        $grade_grade = null;
+        $gradegrade = null;
         $grade       = null;
 
         foreach ($rs as $gd) {
@@ -233,21 +233,21 @@ function grade_update($source, $courseid, $itemtype, $itemmodule, $iteminstance,
             }
             // existing grade requested
             $grade       = $grades[$userid];
-            $grade_grade = new grade_grade($gd, false);
+            $gradegrade = new grade_grade($gd, false);
             unset($grades[$userid]);
             break;
         }
 
-        if (is_null($grade_grade)) {
+        if (is_null($gradegrade)) {
             if (count($grades) == 0) {
-                // no more grades to process
+                // No more grades to process.
                 break;
             }
 
             $grade       = reset($grades);
             $userid      = $grade['userid'];
-            $grade_grade = new grade_grade(array('itemid'=>$grade_item->id, 'userid'=>$userid), false);
-            $grade_grade->load_optional_fields(); // add feedback and info too
+            $gradegrade = new grade_grade(array('itemid' => $gradeitem->id, 'userid' => $userid), false);
+            $gradegrade->load_optional_fields(); // add feedback and info too
             unset($grades[$userid]);
         }
 
@@ -288,8 +288,8 @@ function grade_update($source, $courseid, $itemtype, $itemmodule, $iteminstance,
         }
 
         // update or insert the grade
-        if (!$grade_item->update_raw_grade($userid, $rawgrade, $source, $feedback, $feedbackformat, $usermodified,
-                $dategraded, $datesubmitted, $grade_grade, $feedbackfiles)) {
+        if (!$gradeitem->update_raw_grade($userid, $rawgrade, $source, $feedback, $feedbackformat, $usermodified,
+                $dategraded, $datesubmitted, $gradegrade, $feedbackfiles, $isbulkupdate)) {
             $failed = true;
         }
     }
@@ -305,6 +305,35 @@ function grade_update($source, $courseid, $itemtype, $itemmodule, $iteminstance,
     }
 }
 
+/**
+ * Returns whether the item is gradable or not. It's considered gradable when there is at least one gradeitem
+ * set as GRADE_TYPE_VALUE or GRADE_TYPE_SCALE.
+ *
+ * @category grade
+ * @param int $courseid ID of course
+ * @param string $itemtype Type of grade item. For example, 'mod' or 'block'
+ * @param string $itemmodule More specific then $itemtype. For example, 'forum' or 'quiz'. May be NULL for some item types
+ * @param int $iteminstance Instance ID of graded item. For example the forum ID.
+ * @return bool returns true if the there is any grade item set as GRADE_TYPE_VALUE, GRADE_TYPE_SCALE.
+ * @category grade
+ */
+function is_gradable(int $courseid, string $itemtype, string $itemmodule, int $iteminstance): bool {
+    $items = grade_item::fetch_all([
+        'itemtype' => $itemtype,
+        'itemmodule' => $itemmodule,
+        'iteminstance' => $iteminstance,
+        'courseid' => $courseid,
+    ]);
+    if ($items) {
+        foreach ($items as $item) {
+            if ($item->gradetype == GRADE_TYPE_VALUE || $item->gradetype == GRADE_TYPE_SCALE) {
+                return true;
+            }
+        }
+    }
+    return false;
+}
+
 /**
  * Updates a user's outcomes. Manual outcomes can not be updated.
  *
@@ -354,6 +383,10 @@ function grade_needs_regrade_final_grades($courseid) {
 function grade_needs_regrade_progress_bar($courseid) {
     global $DB;
     $grade_items = grade_item::fetch_all(array('courseid' => $courseid));
+    if (!$grade_items) {
+        // If there are no grade items then we definitely don't need a progress bar!
+        return false;
+    }
 
     list($sql, $params) = $DB->get_in_or_equal(array_keys($grade_items), SQL_PARAMS_NAMED, 'gi');
     $gradecount = $DB->count_records_select('grade_grades', 'itemid ' . $sql, $params);
@@ -376,44 +409,19 @@ function grade_needs_regrade_progress_bar($courseid) {
  *
  * @param stdClass $course The course to regrade
  * @param callable $callback A function to call if regrading took place
- * @return moodle_url The URL to redirect to if redirecting
+ * @return moodle_url|false The URL to redirect to if redirecting
  */
-function grade_regrade_final_grades_if_required($course, callable $callback = null) {
-    global $PAGE, $OUTPUT;
+function grade_regrade_final_grades_if_required($course, ?callable $callback = null) {
+    global $PAGE;
 
     if (!grade_needs_regrade_final_grades($course->id)) {
         return false;
     }
 
     if (grade_needs_regrade_progress_bar($course->id)) {
-        $PAGE->set_heading($course->fullname);
-        echo $OUTPUT->header();
-        echo $OUTPUT->heading(get_string('recalculatinggrades', 'grades'));
-        $progress = new \core\progress\display(true);
-        $status = grade_regrade_final_grades($course->id, null, null, $progress);
-
-        // Show regrade errors and set the course to no longer needing regrade (stop endless loop).
-        if (is_array($status)) {
-            foreach ($status as $error) {
-                $errortext = new \core\output\notification($error, \core\output\notification::NOTIFY_ERROR);
-                echo $OUTPUT->render($errortext);
-            }
-            $courseitem = grade_item::fetch_course_item($course->id);
-            $courseitem->regrading_finished();
-        }
-
-        if ($callback) {
-            //
-            $url = call_user_func($callback);
-        }
-
-        if (empty($url)) {
-            $url = $PAGE->url;
-        }
-
-        echo $OUTPUT->continue_button($url);
-        echo $OUTPUT->footer();
-        die();
+        // Queue ad-hoc task and redirect.
+        grade_regrade_final_grades($course->id, async: true);
+        return $callback ? call_user_func($callback) : $PAGE->url;
     } else {
         $result = grade_regrade_final_grades($course->id);
         if ($callback) {
@@ -424,16 +432,22 @@ function grade_regrade_final_grades_if_required($course, callable $callback = nu
 }
 
 /**
- * Returns grading information for given activity, optionally with user grades
+ * Returns grading information for given activity, optionally with user grades.
  * Manual, course or category items can not be queried.
  *
- * @category grade
+ * This function can be VERY costly - it is doing full course grades recalculation if needsupdate = 1
+ * for course grade item. So be sure you really need it.
+ * If you need just certain grades consider using grade_item::refresh_grades()
+ * together with grade_item::get_grade() instead.
+ *
  * @param int    $courseid ID of course
  * @param string $itemtype Type of grade item. For example, 'mod' or 'block'
  * @param string $itemmodule More specific then $itemtype. For example, 'forum' or 'quiz'. May be NULL for some item types
  * @param int    $iteminstance ID of the item module
  * @param mixed  $userid_or_ids Either a single user ID, an array of user IDs or null. If user ID or IDs are not supplied returns information about grade_item
- * @return array Array of grade information objects (scaleid, name, grade and locked status, etc.) indexed with itemnumbers
+ * @return stdClass Object with keys {items, outcomes, errors}, where 'items' is an array of grade
+ *               information objects (scaleid, name, grade and locked status, etc.) indexed with itemnumbers
+ * @category grade
  */
 function grade_get_grades($courseid, $itemtype, $itemmodule, $iteminstance, $userid_or_ids=null) {
     global $CFG;
@@ -441,13 +455,20 @@ function grade_get_grades($courseid, $itemtype, $itemmodule, $iteminstance, $use
     $return = new stdClass();
     $return->items    = array();
     $return->outcomes = array();
+    $return->errors = [];
 
-    $course_item = grade_item::fetch_course_item($courseid);
+    $courseitem = grade_item::fetch_course_item($courseid);
     $needsupdate = array();
-    if ($course_item->needsupdate) {
+    if ($courseitem->needsupdate) {
         $result = grade_regrade_final_grades($courseid);
         if ($result !== true) {
             $needsupdate = array_keys($result);
+            // Return regrade errors if the user has capability.
+            $context = context_course::instance($courseid);
+            if (has_capability('moodle/grade:edit', $context)) {
+                $return->errors = $result;
+            }
+            $courseitem->regrading_finished();
         }
     }
 
@@ -513,6 +534,7 @@ function grade_get_grades($courseid, $itemtype, $itemmodule, $iteminstance, $use
                         $grade->usermodified   = $grade_grades[$userid]->usermodified;
                         $grade->datesubmitted  = $grade_grades[$userid]->get_datesubmitted();
                         $grade->dategraded     = $grade_grades[$userid]->get_dategraded();
+                        $grade->deductedmark   = $grade_grades[$userid]->deductedmark;
 
                         // create text representation of grade
                         if ($grade_item->gradetype == GRADE_TYPE_TEXT or $grade_item->gradetype == GRADE_TYPE_NONE) {
@@ -676,16 +698,18 @@ function grade_get_grades($courseid, $itemtype, $itemmodule, $iteminstance, $use
 function grade_get_setting($courseid, $name, $default=null, $resetcache=false) {
     global $DB;
 
-    static $cache = array();
+    $cache = cache::make('core', 'gradesetting');
+    $gradesetting = $cache->get($courseid) ?: array();
 
-    if ($resetcache or !array_key_exists($courseid, $cache)) {
-        $cache[$courseid] = array();
+    if ($resetcache or empty($gradesetting)) {
+        $gradesetting = array();
+        $cache->set($courseid, $gradesetting);
 
     } else if (is_null($name)) {
         return null;
 
-    } else if (array_key_exists($name, $cache[$courseid])) {
-        return $cache[$courseid][$name];
+    } else if (array_key_exists($name, $gradesetting)) {
+        return $gradesetting[$name];
     }
 
     if (!$data = $DB->get_record('grade_settings', array('courseid'=>$courseid, 'name'=>$name))) {
@@ -698,7 +722,8 @@ function grade_get_setting($courseid, $name, $default=null, $resetcache=false) {
         $result = $default;
     }
 
-    $cache[$courseid][$name] = $result;
+    $gradesetting[$name] = $result;
+    $cache->set($courseid, $gradesetting);
     return $result;
 }
 
@@ -756,14 +781,14 @@ function grade_set_setting($courseid, $name, $value) {
 /**
  * Returns string representation of grade value
  *
- * @param float $value The grade value
+ * @param float|null $value The grade value
  * @param object $grade_item Grade item object passed by reference to prevent scale reloading
  * @param bool $localized use localised decimal separator
  * @param int $displaytype type of display. For example GRADE_DISPLAY_TYPE_REAL, GRADE_DISPLAY_TYPE_PERCENTAGE, GRADE_DISPLAY_TYPE_LETTER
  * @param int $decimals The number of decimal places when displaying float values
  * @return string
  */
-function grade_format_gradevalue($value, &$grade_item, $localized=true, $displaytype=null, $decimals=null) {
+function grade_format_gradevalue(?float $value, &$grade_item, $localized=true, $displaytype=null, $decimals=null) {
     if ($grade_item->gradetype == GRADE_TYPE_NONE or $grade_item->gradetype == GRADE_TYPE_TEXT) {
         return '';
     }
@@ -827,13 +852,13 @@ function grade_format_gradevalue($value, &$grade_item, $localized=true, $display
 /**
  * Returns a float representation of a grade value
  *
- * @param float $value The grade value
+ * @param float|null $value The grade value
  * @param object $grade_item Grade item object
  * @param int $decimals The number of decimal places
  * @param bool $localized use localised decimal separator
  * @return string
  */
-function grade_format_gradevalue_real($value, $grade_item, $decimals, $localized) {
+function grade_format_gradevalue_real(?float $value, $grade_item, $decimals, $localized) {
     if ($grade_item->gradetype == GRADE_TYPE_SCALE) {
         if (!$scale = $grade_item->load_scale()) {
             return get_string('error');
@@ -850,13 +875,13 @@ function grade_format_gradevalue_real($value, $grade_item, $decimals, $localized
 /**
  * Returns a percentage representation of a grade value
  *
- * @param float $value The grade value
+ * @param float|null $value The grade value
  * @param object $grade_item Grade item object
  * @param int $decimals The number of decimal places
  * @param bool $localized use localised decimal separator
  * @return string
  */
-function grade_format_gradevalue_percentage($value, $grade_item, $decimals, $localized) {
+function grade_format_gradevalue_percentage(?float $value, $grade_item, $decimals, $localized) {
     $min = $grade_item->grademin;
     $max = $grade_item->grademax;
     if ($min == $max) {
@@ -871,11 +896,11 @@ function grade_format_gradevalue_percentage($value, $grade_item, $decimals, $loc
  * Returns a letter grade representation of a grade value
  * The array of grade letters used is produced by {@link grade_get_letters()} using the course context
  *
- * @param float $value The grade value
+ * @param float|null $value The grade value
  * @param object $grade_item Grade item object
  * @return string
  */
-function grade_format_gradevalue_letter($value, $grade_item) {
+function grade_format_gradevalue_letter(?float $value, $grade_item) {
     global $CFG;
     $context = context_course::instance($grade_item->courseid, IGNORE_MISSING);
     if (!$letters = grade_get_letters($context)) {
@@ -1131,9 +1156,10 @@ function grade_recover_history_grades($userid, $courseid) {
  * @param int $userid If specified try to do a quick regrading of the grades of this user only
  * @param object $updated_item Optional grade item to be marked for regrading. It is required if $userid is set.
  * @param \core\progress\base $progress If provided, will be used to update progress on this long operation.
- * @return bool true if ok, array of errors if problems found. Grade item id => error message
+ * @param bool $async If true, and we are recalculating an entire course's grades, defer processing to an ad-hoc task.
+ * @return array|true true if ok, array of errors if problems found. Grade item id => error message
  */
-function grade_regrade_final_grades($courseid, $userid=null, $updated_item=null, $progress=null) {
+function grade_regrade_final_grades($courseid, $userid=null, $updated_item=null, $progress=null, bool $async = false) {
     // This may take a very long time and extra memory.
     \core_php_time_limit::raise();
     raise_memory_limit(MEMORY_EXTRA);
@@ -1147,7 +1173,7 @@ function grade_regrade_final_grades($courseid, $userid=null, $updated_item=null,
     if ($userid) {
         // one raw grade updated for one user
         if (empty($updated_item)) {
-            print_error("cannotbenull", 'debug', '', "updated_item");
+            throw new \moodle_exception("cannotbenull", 'debug', '', "updated_item");
         }
         if ($course_item->needsupdate) {
             $updated_item->force_regrading();
@@ -1157,6 +1183,35 @@ function grade_regrade_final_grades($courseid, $userid=null, $updated_item=null,
     } else {
         if (!$course_item->needsupdate) {
             // nothing to do :-)
+            if ($progress instanceof \core\progress\stored) {
+                // The regrade was already run elsewhere without the stored progress, so just start and end it now.
+                $progress->start_progress(get_string('recalculatinggrades', 'grades'));
+                $progress->end_progress();
+            }
+            return true;
+        }
+        // Defer recalculation to an ad-hoc task.
+        if ($async) {
+            $regradecache = cache::make_from_params(
+                mode: cache_store::MODE_REQUEST,
+                component: 'core',
+                area: 'grade_regrade_final_grades',
+                options: [
+                    'simplekeys' => true,
+                    'simpledata' => true,
+                ],
+            );
+            // If the courseid already exists in the cache, return so we don't do this multiple times per request.
+            if ($regradecache->get($courseid)) {
+                return true;
+            }
+            $task = \core_course\task\regrade_final_grades::create($courseid);
+            $taskid = \core\task\manager::queue_adhoc_task($task, true);
+            if ($taskid) {
+                $task->set_id($taskid);
+                $task->initialise_stored_progress();
+            }
+            $regradecache->set($courseid, true);
             return true;
         }
     }
@@ -1199,7 +1254,7 @@ function grade_regrade_final_grades($courseid, $userid=null, $updated_item=null,
         }
     }
 
-    $progress->start_progress('regrade_course', $progresstotal);
+    $progress->start_progress(get_string('recalculatinggrades', 'grades'), $progresstotal);
 
     $errors = array();
     $finalids = array();
@@ -1270,7 +1325,7 @@ function grade_regrade_final_grades($courseid, $userid=null, $updated_item=null,
             }
 
             // Let's update, calculate or aggregate.
-            $result = $grade_items[$gid]->regrade_final_grades($userid);
+            $result = $grade_items[$gid]->regrade_final_grades($userid, $progress);
 
             if ($result === true) {
 
@@ -1304,7 +1359,10 @@ function grade_regrade_final_grades($courseid, $userid=null, $updated_item=null,
                     continue; // this one is ok
                 }
                 $grade_items[$gid]->force_regrading();
-                $errors[$grade_items[$gid]->id] = get_string('errorcalculationbroken', 'grades');
+                if (!empty($grade_items[$gid]->calculation) && empty($errors[$gid])) {
+                    $itemname = $grade_items[$gid]->get_name();
+                    $errors[$gid] = get_string('errorcalculationbroken', 'grades', $itemname);
+                }
             }
             break; // Found error.
         }
@@ -1347,7 +1405,7 @@ function grade_grab_course_grades($courseid, $modname=null, $userid=0) {
     }
 
     if (!$mods = core_component::get_plugin_list('mod') ) {
-        print_error('nomodules', 'debug');
+        throw new \moodle_exception('nomodules', 'debug');
     }
 
     foreach ($mods as $mod => $fullmod) {
@@ -1576,19 +1634,19 @@ function grade_course_reset($courseid) {
     grade_grab_course_grades($courseid);
 
     // recalculate all grades
-    grade_regrade_final_grades($courseid);
+    grade_regrade_final_grades($courseid, async: true);
     return true;
 }
 
 /**
- * Convert a number to 5 decimal point float, an empty string or a null db compatible format
+ * Convert a number to 5 decimal point float, null db compatible format
  * (we need this to decide if db value changed)
  *
- * @param mixed $number The number to convert
- * @return mixed float or null
+ * @param float|null $number The number to convert
+ * @return float|null float or null
  */
-function grade_floatval($number) {
-    if (is_null($number) or $number === '') {
+function grade_floatval(?float $number) {
+    if (is_null($number)) {
         return null;
     }
     // we must round to 5 digits to get the same precision as in 10,5 db fields
@@ -1600,11 +1658,11 @@ function grade_floatval($number) {
  * Compare two float numbers safely. Uses 5 decimals php precision using {@link grade_floatval()}. Nulls accepted too.
  * Used for determining if a database update is required
  *
- * @param float $f1 Float one to compare
- * @param float $f2 Float two to compare
+ * @param float|null $f1 Float one to compare
+ * @param float|null $f2 Float two to compare
  * @return bool True if the supplied values are different
  */
-function grade_floats_different($f1, $f2) {
+function grade_floats_different(?float $f1, ?float $f2): bool {
     // note: db rounding for 10,5 is different from php round() function
     return (grade_floatval($f1) !== grade_floatval($f2));
 }
@@ -1616,11 +1674,11 @@ function grade_floats_different($f1, $f2) {
  * different from php round() function.
  *
  * @since Moodle 2.0
- * @param float $f1 Float one to compare
- * @param float $f2 Float two to compare
+ * @param float|null $f1 Float one to compare
+ * @param float|null $f2 Float two to compare
  * @return bool True if the values should be considered as the same grades
  */
-function grade_floats_equal($f1, $f2) {
+function grade_floats_equal(?float $f1, ?float $f2): bool {
     return (grade_floatval($f1) === grade_floatval($f2));
 }
 
