@@ -43,6 +43,13 @@ class report extends \mod_scorm\report {
         $attemptsmode = optional_param('attemptsmode', SCORM_REPORT_ATTEMPTS_ALL_STUDENTS, PARAM_INT);
         $PAGE->set_url(new \moodle_url($PAGE->url, array('attemptsmode' => $attemptsmode)));
 
+        // Scorm action bar for report.
+        if ($download === '') {
+            $actionbar = new \mod_scorm\output\actionbar($cm->id, true, $attemptsmode);
+            $renderer = $PAGE->get_renderer('mod_scorm');
+            echo $renderer->report_actionbar($actionbar);
+        }
+
         if ($action == 'delete' && has_capability('mod/scorm:deleteresponses', $contextmodule) && confirm_sesskey()) {
             if (scorm_delete_responses($attemptids, $scorm)) { // Delete responses.
                 echo $OUTPUT->notification(get_string('scormresponsedeleted', 'scorm'), 'notifysuccess');
@@ -117,10 +124,11 @@ class report extends \mod_scorm\report {
             }
             $columns[] = 'fullname';
             $headers[] = get_string('name');
-            $extrafields = get_extra_user_fields($coursecontext);
+            // TODO Does not support custom user profile fields (MDL-70456).
+            $extrafields = \core_user\fields::get_identity_fields($coursecontext, false);
             foreach ($extrafields as $field) {
                 $columns[] = $field;
-                $headers[] = get_user_field_name($field);
+                $headers[] = \core_user\fields::get_display_name($field);
             }
 
             $columns[] = 'attempt';
@@ -266,31 +274,32 @@ class report extends \mod_scorm\report {
                 $csvexport->add_data($headers);
             }
             // Construct the SQL.
-            $select = 'SELECT DISTINCT '.$DB->sql_concat('u.id', '\'#\'', 'COALESCE(st.attempt, 0)').' AS uniqueid, ';
-            $select .= 'st.scormid AS scormid, st.attempt AS attempt, ' .
-                    \user_picture::fields('u', array('idnumber'), 'userid') .
-                    get_extra_user_fields_sql($coursecontext, 'u', '', array('email', 'idnumber')) . ' ';
+            $select = 'SELECT DISTINCT '.$DB->sql_concat('u.id', '\'#\'', 'COALESCE(sa.attempt, 0)').' AS uniqueid, ';
+            // TODO Does not support custom user profile fields (MDL-70456).
+            $userfields = \core_user\fields::for_identity($coursecontext, false)->with_userpic()->including('idnumber');
+            $selectfields = $userfields->get_sql('u', false, '', 'userid')->selects;
+            $select .= 'sa.scormid AS scormid, sa.attempt AS attempt ' . $selectfields . ' ';
 
-            // This part is the same for all cases - join users and scorm_scoes_track tables.
+            // This part is the same for all cases - join users and user tracking tables.
             $from = 'FROM {user} u ';
-            $from .= 'LEFT JOIN {scorm_scoes_track} st ON st.userid = u.id AND st.scormid = '.$scorm->id;
+            $from .= 'LEFT JOIN {scorm_attempt} sa ON sa.userid = u.id AND sa.scormid = '.$scorm->id;
             switch ($attemptsmode) {
                 case SCORM_REPORT_ATTEMPTS_STUDENTS_WITH:
                     // Show only students with attempts.
-                    $where = " WHERE u.id IN ({$allowedlistsql}) AND st.userid IS NOT NULL";
+                    $where = " WHERE u.id IN ({$allowedlistsql}) AND sa.userid IS NOT NULL";
                     break;
                 case SCORM_REPORT_ATTEMPTS_STUDENTS_WITH_NO:
                     // Show only students without attempts.
-                    $where = " WHERE u.id IN ({$allowedlistsql}) AND st.userid IS NULL";
+                    $where = " WHERE u.id IN ({$allowedlistsql}) AND sa.userid IS NULL";
                     break;
                 case SCORM_REPORT_ATTEMPTS_ALL_STUDENTS:
                     // Show all students with or without attempts.
-                    $where = " WHERE u.id IN ({$allowedlistsql}) AND (st.userid IS NOT NULL OR st.userid IS NULL)";
+                    $where = " WHERE u.id IN ({$allowedlistsql}) AND (sa.userid IS NOT NULL OR sa.userid IS NULL)";
                     break;
             }
 
-            $countsql = 'SELECT COUNT(DISTINCT('.$DB->sql_concat('u.id', '\'#\'', 'COALESCE(st.attempt, 0)').')) AS nbresults, ';
-            $countsql .= 'COUNT(DISTINCT('.$DB->sql_concat('u.id', '\'#\'', 'st.attempt').')) AS nbattempts, ';
+            $countsql = 'SELECT COUNT(DISTINCT('.$DB->sql_concat('u.id', '\'#\'', 'COALESCE(sa.attempt, 0)').')) AS nbresults, ';
+            $countsql .= 'COUNT(DISTINCT('.$DB->sql_concat('u.id', '\'#\'', 'sa.attempt').')) AS nbattempts, ';
             $countsql .= 'COUNT(DISTINCT(u.id)) AS nbusers ';
             $countsql .= $from.$where;
 
@@ -377,7 +386,7 @@ class report extends \mod_scorm\report {
                     }
                     if (in_array('picture', $columns)) {
                         $user = new \stdClass();
-                        $additionalfields = explode(',', \user_picture::fields());
+                        $additionalfields = explode(',', implode(',', \core_user\fields::get_picture_fields()));
                         $user = username_load_fields_from_object($user, $scouser, null, $additionalfields);
                         $user->id = $scouser->userid;
                         $row[] = $OUTPUT->user_picture($user, array('courseid' => $course->id));
@@ -399,7 +408,7 @@ class report extends \mod_scorm\report {
                     } else {
                         if (!$download) {
                             $url = new \moodle_url('/mod/scorm/report/userreport.php', array('id' => $cm->id,
-                                    'user' => $scouser->userid, 'attempt' => $scouser->attempt));
+                                'user' => $scouser->userid, 'attempt' => $scouser->attempt, 'mode' => 'basic'));
                             $row[] = \html_writer::link($url, $scouser->attempt);
                         } else {
                             $row[] = $scouser->attempt;
@@ -443,7 +452,8 @@ class report extends \mod_scorm\report {
                                     }
                                     if (!$download) {
                                         $url = new \moodle_url('/mod/scorm/report/userreporttracks.php', array('id' => $cm->id,
-                                            'scoid' => $sco->id, 'user' => $scouser->userid, 'attempt' => $scouser->attempt));
+                                            'scoid' => $sco->id, 'user' => $scouser->userid, 'attempt' => $scouser->attempt,
+                                            'mode' => 'basic'));
                                         $row[] = $OUTPUT->pix_icon($trackdata->status, $strstatus, 'scorm') . '<br>' .
                                            \html_writer::link($url, $score, array('title' => get_string('details', 'scorm')));
                                     } else {
@@ -478,7 +488,7 @@ class report extends \mod_scorm\report {
                 if (!$download) {
                     $table->finish_output();
                     if ($candelete) {
-                        echo \html_writer::start_tag('table', array('id' => 'commands'));
+                        echo \html_writer::start_tag('table', ['id' => 'commands', 'class' => 'table-reboot']);
                         echo \html_writer::start_tag('tr').\html_writer::start_tag('td');
                         echo $this->generate_delete_selected_button();
                         echo \html_writer::end_tag('td').\html_writer::end_tag('tr').\html_writer::end_tag('table');
@@ -486,28 +496,6 @@ class report extends \mod_scorm\report {
                         echo \html_writer::end_tag('div');
                         echo \html_writer::end_tag('form');
                     }
-                    echo \html_writer::end_div();
-                    if (!empty($attempts)) {
-                        echo \html_writer::start_tag('table', array('class' => 'boxaligncenter')).\html_writer::start_tag('tr');
-                        echo \html_writer::start_tag('td');
-                        echo $OUTPUT->single_button(new \moodle_url($PAGE->url,
-                                                                   array('download' => 'ODS') + $displayoptions),
-                                                                   get_string('downloadods'));
-                        echo \html_writer::end_tag('td');
-                        echo \html_writer::start_tag('td');
-                        echo $OUTPUT->single_button(new \moodle_url($PAGE->url,
-                                                                   array('download' => 'Excel') + $displayoptions),
-                                                                   get_string('downloadexcel'));
-                        echo \html_writer::end_tag('td');
-                        echo \html_writer::start_tag('td');
-                        echo $OUTPUT->single_button(new \moodle_url($PAGE->url,
-                                                                   array('download' => 'CSV') + $displayoptions),
-                                                                   get_string('downloadtext'));
-                        echo \html_writer::end_tag('td');
-                        echo \html_writer::start_tag('td');
-                        echo \html_writer::end_tag('td');
-                        echo \html_writer::end_tag('tr').\html_writer::end_tag('table');
-                    }
                 }
             } else {
                 if ($candelete && !$download) {
