@@ -26,11 +26,11 @@
 defined('MOODLE_INTERNAL') || die();
 
 $plugin->component = 'qbehaviour_deferredcbm';
-$plugin->version   = 2022041900;
+$plugin->version   = 2025041400;
 
-$plugin->requires  = 2022041200;
-$plugin->dependencies = array(
-    'qbehaviour_deferredfeedback' => 2022041200
-);
+$plugin->requires  = 2025040800;
+$plugin->dependencies = [
+    'qbehaviour_deferredfeedback' => 2025040800,
+];
 
 $plugin->maturity  = MATURITY_STABLE;
