@@ -18,11 +18,18 @@ declare(strict_types=1);
 
 namespace core_reportbuilder\local\entities;
 
+use context_helper;
 use context_system;
+use context_user;
+use core\context;
+use core_component;
+use core_date;
+use core_user;
 use html_writer;
 use lang_string;
 use moodle_url;
 use stdClass;
+use theme_config;
 use core_user\fields;
 use core_reportbuilder\local\filters\boolean_select;
 use core_reportbuilder\local\filters\date;
@@ -46,12 +53,17 @@ use core_reportbuilder\local\report\filter;
 class user extends base {
 
     /**
-     * Database tables that this entity uses and their default aliases
+     * Database tables that this entity uses
      *
-     * @return array
+     * @return string[]
      */
-    protected function get_default_table_aliases(): array {
-        return ['user' => 'u'];
+    protected function get_default_tables(): array {
+        return [
+            'user',
+            'context',
+            'tag_instance',
+            'tag',
+        ];
     }
 
     /**
@@ -64,40 +76,98 @@ class user extends base {
     }
 
     /**
-     * Initialise the entity, add all user fields and all 'visible' user profile fields
+     * Initialise the entity
      *
      * @return base
      */
     public function initialise(): base {
-        $userprofilefields = $this->get_user_profile_fields();
+        $tablealias = $this->get_table_alias('user');
+
+        $userprofilefields = (new user_profile_fields(
+            "{$tablealias}.id",
+            $this->get_entity_name(),
+        ))
+            ->add_joins($this->get_joins());
 
         $columns = array_merge($this->get_all_columns(), $userprofilefields->get_columns());
         foreach ($columns as $column) {
             $this->add_column($column);
         }
 
+        // All the filters defined by the entity can also be used as conditions.
         $filters = array_merge($this->get_all_filters(), $userprofilefields->get_filters());
         foreach ($filters as $filter) {
-            $this->add_filter($filter);
-        }
-
-        $conditions = array_merge($this->get_all_filters(), $userprofilefields->get_filters());
-        foreach ($conditions as $condition) {
-            $this->add_condition($condition);
+            $this
+                ->add_condition($filter)
+                ->add_filter($filter);
         }
 
         return $this;
     }
 
     /**
-     * Get user profile fields helper instance
+     * Returns column that corresponds to the given identity field, profile field identifiers will be converted to those
+     * used by the {@see user_profile_fields} helper
      *
-     * @return user_profile_fields
+     * @param string $identityfield Field from the user table, or a custom profile field
+     * @return column
      */
-    protected function get_user_profile_fields(): user_profile_fields {
-        $userprofilefields = new user_profile_fields($this->get_table_alias('user') . '.id', $this->get_entity_name());
-        $userprofilefields->add_joins($this->get_joins());
-        return $userprofilefields;
+    public function get_identity_column(string $identityfield): column {
+        if (preg_match(fields::PROFILE_FIELD_REGEX, $identityfield, $matches)) {
+            $identityfield = 'profilefield_' . $matches[1];
+        }
+
+        return $this->get_column($identityfield);
+    }
+
+    /**
+     * Returns columns that correspond to the site configured identity fields
+     *
+     * @param context $context
+     * @param string[] $excluding
+     * @return column[]
+     */
+    public function get_identity_columns(context $context, array $excluding = []): array {
+        $identityfields = fields::for_identity($context)->excluding(...$excluding)->get_required_fields();
+
+        return array_map([$this, 'get_identity_column'], $identityfields);
+    }
+
+    /**
+     * Returns filter that corresponds to the given identity field, profile field identifiers will be converted to those
+     * used by the {@see user_profile_fields} helper
+     *
+     * @param string $identityfield Field from the user table, or a custom profile field
+     * @return filter
+     */
+    public function get_identity_filter(string $identityfield): filter {
+        if (preg_match(fields::PROFILE_FIELD_REGEX, $identityfield, $matches)) {
+            $identityfield = 'profilefield_' . $matches[1];
+        }
+
+        return $this->get_filter($identityfield);
+    }
+
+    /**
+     * Returns filters that correspond to the site configured identity fields
+     *
+     * @param context $context
+     * @param string[] $excluding
+     * @return filter[]
+     */
+    public function get_identity_filters(context $context, array $excluding = []): array {
+        $identityfields = fields::for_identity($context)->excluding(...$excluding)->get_required_fields();
+
+        return array_map([$this, 'get_identity_filter'], $identityfields);
+    }
+
+    /**
+     * Return joins necessary for retrieving tags
+     *
+     * @return string[]
+     */
+    public function get_tag_joins(): array {
+        return $this->get_tag_joins_for_entity('core', 'user', $this->get_table_alias('user') . '.id');
     }
 
     /**
@@ -108,7 +178,10 @@ class user extends base {
      * @return column[]
      */
     protected function get_all_columns(): array {
+        global $DB;
+
         $usertablealias = $this->get_table_alias('user');
+        $contexttablealias = $this->get_table_alias('context');
 
         $fullnameselect = self::get_name_fields_select($usertablealias);
         $fullnamesort = explode(', ', $fullnameselect);
@@ -124,10 +197,11 @@ class user extends base {
         ))
             ->add_joins($this->get_joins())
             ->add_fields($fullnameselect)
-            ->set_type(column::TYPE_TEXT)
-            ->set_is_sortable($this->is_sortable('fullname'), $fullnamesort)
-            ->add_callback(static function(?string $value, stdClass $row) use ($viewfullnames): string {
-                if ($value === null) {
+            ->set_is_sortable(true, $fullnamesort)
+            ->add_callback(static function($value, stdClass $row) use ($viewfullnames): string {
+
+                // Ensure we have at least one field present.
+                if (count(array_filter((array) $row, fn($field) => $field !== null)) === 0) {
                     return '';
                 }
 
@@ -155,12 +229,12 @@ class user extends base {
                 ->add_joins($this->get_joins())
                 ->add_fields($fullnameselect)
                 ->add_field("{$usertablealias}.id")
-                ->set_type(column::TYPE_TEXT)
-                ->set_is_sortable($this->is_sortable($fullnamefield), $fullnamesort)
-                ->add_callback(static function(?string $value, stdClass $row) use ($fullnamefield, $viewfullnames): string {
+                ->set_is_sortable(true, $fullnamesort)
+                ->add_callback(static function($value, stdClass $row) use ($fullnamefield, $viewfullnames): string {
                     global $OUTPUT;
 
-                    if ($value === null) {
+                    // Ensure we have at least one field present.
+                    if (count(array_filter((array) $row, fn($field) => $field !== null)) === 0) {
                         return '';
                     }
 
@@ -184,7 +258,7 @@ class user extends base {
                             fullname($row, $viewfullnames));
                     }
 
-                    return $value;
+                    return (string) $value;
                 });
 
             // Picture fields need some more data.
@@ -198,16 +272,12 @@ class user extends base {
         // Picture column.
         $columns[] = (new column(
             'picture',
-            new lang_string('userpicture', 'core_reportbuilder'),
+            new lang_string('picture'),
             $this->get_entity_name()
         ))
             ->add_joins($this->get_joins())
             ->add_fields($userpictureselect)
-            ->set_type(column::TYPE_INTEGER)
-            ->set_is_sortable($this->is_sortable('picture'))
-            // It doesn't make sense to offer integer aggregation methods for this column.
-            ->set_disabled_aggregation(['avg', 'max', 'min', 'sum'])
-            ->add_callback(static function ($value, stdClass $row): string {
+            ->add_callback(static function($value, stdClass $row): string {
                 global $OUTPUT;
 
                 return !empty($row->id) ? $OUTPUT->user_picture($row, ['link' => false, 'alttext' => false]) : '';
@@ -224,17 +294,19 @@ class user extends base {
                 $this->get_entity_name()
             ))
                 ->add_joins($this->get_joins())
-                ->add_field("{$usertablealias}.{$userfield}")
                 ->set_type($columntype)
-                ->set_is_sortable($this->is_sortable($userfield))
+                ->add_field("{$usertablealias}.{$userfield}")
+                ->set_is_sortable(true)
                 ->add_callback([$this, 'format'], $userfield);
 
-            // Some columns also have specific format callbacks.
-            if ($userfield === 'country') {
-                $column->add_callback(static function(string $country): string {
-                    $countries = get_string_manager()->get_list_of_countries(true);
-                    return $countries[$country] ?? '';
-                });
+            // Join on the context table so that we can use it for formatting these columns later.
+            if ($userfield === 'description') {
+                $column
+                    ->add_join("LEFT JOIN {context} {$contexttablealias}
+                           ON {$contexttablealias}.contextlevel = " . CONTEXT_USER . "
+                          AND {$contexttablealias}.instanceid = {$usertablealias}.id")
+                    ->add_fields("{$usertablealias}.descriptionformat, {$usertablealias}.id")
+                    ->add_fields(context_helper::get_preload_record_columns_sql($contexttablealias));
             }
 
             $columns[] = $column;
@@ -243,21 +315,6 @@ class user extends base {
         return $columns;
     }
 
-    /**
-     * Check if this field is sortable
-     *
-     * @param string $fieldname
-     * @return bool
-     */
-    protected function is_sortable(string $fieldname): bool {
-        // Some columns can't be sorted, like longtext or images.
-        $nonsortable = [
-            'picture',
-        ];
-
-        return !in_array($fieldname, $nonsortable);
-    }
-
     /**
      * Formats the user field for display.
      *
@@ -267,6 +324,8 @@ class user extends base {
      * @return string
      */
     public function format($value, stdClass $row, string $fieldname): string {
+        global $CFG;
+
         if ($this->get_user_field_type($fieldname) === column::TYPE_BOOLEAN) {
             return format::boolean_as_text($value);
         }
@@ -275,6 +334,26 @@ class user extends base {
             return format::userdate($value, $row);
         }
 
+        // If the column has corresponding filter, determine the value from its options.
+        $options = $this->get_options_for($fieldname);
+        if ($options !== null && $value !== null && array_key_exists($value, $options)) {
+            return $options[$value];
+        }
+
+        if ($fieldname === 'description') {
+            if (empty($row->id)) {
+                return '';
+            }
+
+            require_once("{$CFG->libdir}/filelib.php");
+
+            context_helper::preload_from_record($row);
+            $context = context_user::instance($row->id);
+
+            $description = file_rewrite_pluginfile_urls($value, 'pluginfile.php', $context->id, 'user', 'profile', null);
+            return format_text($description, $row->descriptionformat, ['context' => $context->id]);
+        }
+
         return s($value);
     }
 
@@ -290,9 +369,8 @@ class user extends base {
 
         $namefields = fields::get_name_fields(true);
 
-        // Create a dummy user object containing all name fields.
-        $dummyuser = (object) array_combine($namefields, $namefields);
-        $dummyfullname = fullname($dummyuser, true);
+        $viewfullnames = has_capability('moodle/site:viewfullnames', context_system::instance());
+        $dummyfullname = core_user::get_dummy_fullname(null, ['override' => $viewfullnames]);
 
         // Extract any name fields from the fullname format in the order that they appear.
         $matchednames = array_values(order_in_string($namefields, $dummyfullname));
@@ -320,6 +398,10 @@ class user extends base {
             'email' => new lang_string('email'),
             'city' => new lang_string('city'),
             'country' => new lang_string('country'),
+            'lang' => new lang_string('language'),
+            'timezone' => new lang_string('timezone'),
+            'theme' => new lang_string('theme'),
+            'description' => new lang_string('description'),
             'firstnamephonetic' => new lang_string('firstnamephonetic'),
             'lastnamephonetic' => new lang_string('lastnamephonetic'),
             'middlename' => new lang_string('middlename'),
@@ -334,7 +416,11 @@ class user extends base {
             'suspended' => new lang_string('suspended'),
             'confirmed' => new lang_string('confirmed', 'admin'),
             'username' => new lang_string('username'),
+            'auth' => new lang_string('authentication', 'moodle'),
             'moodlenetprofile' => new lang_string('moodlenetprofile', 'user'),
+            'timecreated' => new lang_string('timecreated', 'core_reportbuilder'),
+            'timemodified' => new lang_string('timemodified', 'core_reportbuilder'),
+            'lastip' => new lang_string('lastip'),
         ];
     }
 
@@ -346,11 +432,16 @@ class user extends base {
      */
     protected function get_user_field_type(string $userfield): int {
         switch ($userfield) {
+            case 'description':
+                $fieldtype = column::TYPE_LONGTEXT;
+                break;
             case 'confirmed':
             case 'suspended':
                 $fieldtype = column::TYPE_BOOLEAN;
                 break;
             case 'lastaccess':
+            case 'timecreated':
+            case 'timemodified':
                 $fieldtype = column::TYPE_TIMESTAMP;
                 break;
             default:
@@ -367,7 +458,6 @@ class user extends base {
      * @return filter[]
      */
     protected function get_all_filters(): array {
-        $filters = [];
         $tablealias = $this->get_table_alias('user');
 
         // Fullname filter.
@@ -383,6 +473,16 @@ class user extends base {
         ))
             ->add_joins($this->get_joins());
 
+        // Picture filter.
+        $filters[] = (new filter(
+            boolean_select::class,
+            'picture',
+            new lang_string('picture'),
+            $this->get_entity_name(),
+            "CASE WHEN {$tablealias}.picture > 0 THEN 1 ELSE 0 END",
+        ))
+            ->add_joins($this->get_joins());
+
         // User fields filters.
         $fields = $this->get_user_fields();
         foreach ($fields as $field => $name) {
@@ -402,7 +502,7 @@ class user extends base {
                 $field,
                 $name,
                 $this->get_entity_name(),
-                $tablealias . '.' . $field
+                "{$tablealias}.{$field}"
             ))
                 ->add_joins($this->get_joins());
 
@@ -427,12 +527,75 @@ class user extends base {
         return $filters;
     }
 
+    /**
+     * Gets list of options if the filter supports it
+     *
+     * @param string $fieldname
+     * @return null|array
+     */
+    protected function get_options_for(string $fieldname): ?array {
+        static $cached = [];
+        if (!array_key_exists($fieldname, $cached)) {
+            $callable = [static::class, 'get_options_for_' . $fieldname];
+            if (is_callable($callable)) {
+                $cached[$fieldname] = $callable();
+            } else {
+                $cached[$fieldname] = null;
+            }
+        }
+        return $cached[$fieldname];
+    }
+
+    /**
+     * List of options for the field auth
+     *
+     * @return string[]
+     */
+    public static function get_options_for_auth(): array {
+        $authlist = array_keys(core_component::get_plugin_list('auth'));
+
+        return array_map(
+            fn(string $auth) => get_auth_plugin($auth)->get_title(),
+            array_combine($authlist, $authlist),
+        );
+    }
+
     /**
      * List of options for the field country.
      *
      * @return string[]
      */
     public static function get_options_for_country(): array {
-        return array_map('shorten_text', get_string_manager()->get_list_of_countries());
+        return get_string_manager()->get_list_of_countries();
+    }
+
+    /**
+     * List of options for the field lang.
+     *
+     * @return string[]
+     */
+    public static function get_options_for_lang(): array {
+        return get_string_manager()->get_list_of_translations();
+    }
+
+    /**
+     * List of options for the field timezone.
+     *
+     * @return string[]
+     */
+    public static function get_options_for_timezone(): array {
+        return core_date::get_list_of_timezones(null, true);
+    }
+
+    /**
+     * List of options for the field theme.
+     *
+     * @return string[]
+     */
+    public static function get_options_for_theme(): array {
+        return array_map(
+            fn(theme_config $theme) => $theme->get_theme_name(),
+            get_list_of_themes(),
+        );
     }
 }
